function [lhood,bestrr] = model_match_chroma(model, data)
% [lhood,bestrr] = model_match_chroma(model, data)
%    Evaluate how well data fits model, returning total log likelihood.
%    All possible row-rotations (chroma transpositions) of data are tried, 
%    best is returned.
%    Data can be a cell array, in which case lhood sums up all fields, but 
%    bestrr is a vector.
%    Data is assumed to have one feature vector per COLUMN (sgram-style).
% 2007-04-04

% nchrom dictates equivalence bands for rotating chroma
nchrom = 12;

% option NOT to use covariance
useMeansOnly = 0;

if iscell(data)

  % Branch 1: if data is a cell array, decompose and recurse
  
  lhood = 0;
  for i = 1:length(data)
    [lh,rr] = model_match_chroma(model,data{i});
    lhood = lhood + lh;
    bestrr(i) = rr;
  end
  
else
  
  lhoods = [];
  rrs = 0:(nchrom-1);
  
  for rrsix = 1:length(rrs)
    
    rdata = chromrot2(data,rrs(rrsix),nchrom);

    if useMeansOnly == 1
      % Using means only + cosine distance
      mm = model.mean/norm(model.mean);
      dm = mean(rdata,2);
      dm = dm/norm(dm);
      lhoods(rrsix) = (dm'*mm);

    else
      
      % gmm?
      if size(model.mean,2) == 1
        lhoods(rrsix) = mean(gaussian_prob_invC(rdata,model.mean,model.invsigma,1));
      else
        lhoods(rrsix) = mean(log(sum(mixgauss_prob(rdata,model.mean,model.sigma,model.prior))));
      end

    end
      
  end
  [lhood,bestrrix] = max(lhoods);
  bestrr = rrs(bestrrix);
end
