function K = rewrite_key_ests_cov(FL,M,cheat)
% K = rewrite_key_ests_cov(FL,M,cheat)
%   (Re) write key estimates into existing chroma data files.
%   FL is a list of files of chroma; (-100 chroma file names are
%   derived too)
%   Dual-stream Gaussian key model is in M.
%   Find transposition that maximizes match.
%   Return vector of actual key assignments.
%   "cheat" optionally names a file with ground truth key information.
% 2009-09-19 Dan Ellis dpwe@ee.columbia.edu

if isstr(FL)
  FL = listfileread(FL);
end

if nargin < 3
  cheat = '';
end

if length(cheat) > 0
  disp(['** cheat = ',cheat])
end

if exist(cheat,'file') > 0
  [skey,sname] = textread(cheat,'%s %s');
  skey = keyname2num(skey);
  cheating = 1;
  disp(['Keys read from ',cheat]);
else
  cheating = 0;
  if length(cheat) > 0
    disp(['***WARN: unable to read keys ground truth file ', ...
          cheat]);
  end
end


nf = length(FL);

K = zeros(1,nf);

for i = 1:nf;
  fn = FL{i};
  % Construct -100 name from -400 name
  fn2 = fn;
  fn2(max(find(fn2=='4'))) = '1';

  D=loadftrs_mirex({fn},0);
  
  if cheating == 1
    [p,n,e] = fileparts(fn);
    if n(end-3) == '-'
      n = n(1:end-4);
    end
    [pp,np,ep] = fileparts(p);
    n = fullfile(np,n);
    ix = strmatch(n,sname);
    if length(ix) > 0
      kkey = skey(ix(1));
      disp(['rewrite_key_ests_cov: cheat: ',n,' matched with key ', num2str(kkey)]);
    else
      kkey = 0;
      disp(['rewrite_key_ests_cov: cheat: no match for ',n]);
    end
    
  else
    [ll,rr] = model_match_chroma(M,D);
    kkey = rr(1);
  end
    
  % Write the data out
  load(fn);
  key = kkey;
  save(fn,'ifname','F','bts','tmean','tsd','fctr','fsd','vsn','desc', ...
       'key');

  load(fn2);
  key = kkey;
  save(fn2,'ifname','F','bts','tmean','tsd','fctr','fsd','vsn','desc', ...
       'key');

  [p,n,e] = fileparts(fn);
  disp([datestr(rem(now,1),'HH:MM:SS'), ' song ',num2str(i), ' ', n,' key=',num2str(key)]);
  K(i) = key;
end
